@@ -32,6 +32,7 @@ ICON_EXPAND = os.path.join(DIRNAME, "resources/iconmonstr-arrow-80-16.png")
 IMG_WORLD = os.path.join(DIRNAME, "resources/world.png")
 
 GITHUB_URL = "https://github.com/semuconsulting/PyGPSClient"
+PYPI_URL = "https://pypi.org/project/PyGPSClient/"
 XML_HDR = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
 GPX_NS = " ".join(
     (
@@ -51,6 +52,9 @@ MAX_SNR = 60  # upper limit of graphview snr axis
 DEVICE_ACCURACY = 2.5  # nominal GPS device accuracy (CEP) in meters
 HDOP_RATIO = 20  # arbitrary calibration of accuracy against HDOP
 MAXLOGLINES = 10000  # maximum number of 'lines' per datalog file
+# default error handling behaviour for UBXReader.read() calls
+# 0 (ERR_IGNORE) = ignore errors, 1 (ERR_LOG) - log errors, 2 (ERR_RAISE) = raise errors
+QUITONERRORDEFAULT = 1
 PORTIDS = ("0 I2C", "1 UART1", "2 UART2", "3 USB", "4 SPI")
 ANTSTATUS = ("INIT", "DONTKNOW", "OK", "SHORT", "OPEN")
 ANTPOWER = ("OFF", "ON", "DONTKNOW")
@@ -81,14 +85,12 @@ KNOWNGPS = (
 )
 # list of available bps rates (first entry in list is the default):
 BPSRATES = (9600, 19200, 38400, 57600, 115200, 230400, 460800, 921600, 4800)
-# displayed protocol flags:
-NMEA_PROTOCOL = 0
-UBX_PROTOCOL = 1
-MIXED_PROTOCOL = 2
 # display formats
-PARSED = 0
-BIN = 1
-HEX = 2
+FORMAT_PARSED = 1
+FORMAT_BIN = 2
+FORMAT_HEX = 4
+FORMAT_HEXTABLE = 8
+FORMATS = ("Parsed", "Binary", "Hex String", "Hex Tabular")
 # connection type flags:
 DISCONNECTED = 0
 CONNECTED = 1
@@ -136,8 +138,12 @@ GNSS_LIST = {
     6: ("GLO", "indianred"),
 }
 
-# List of tags to highlight in console
+# List of tags to highlight in console if TAG_COLORS = True
 # (NB there is a slight performance hit in having many tags)
+FONT_MENU = "TkMenuFont"
+FONT_TEXT = "TkTextFont"
+FONT_FIXED = "TkFixedFont"
+TAG_COLORS = True
 TAGS = [
     ("ACK-ACK", "green2"),
     ("ACK-NAK", "orange red"),
@@ -209,4 +215,5 @@ TAGS = [
     ("VLW", "deepskyblue"),
     ("VTG", "deepskyblue"),
     ("ZDA", "cyan"),
+    ("UNKNOWN PROTOCOL", "red"),
 ]
