@@ -14,17 +14,16 @@ Created on 16 Sep 2020
 from io import BufferedReader
 from threading import Thread
 from serial import Serial, SerialException, SerialTimeoutException
+from pyubx2 import UBXReader, UBXParseError, protocol
+from pynmeagps import NMEAParseError
 import pyubx2.ubxtypes_core as ubt
-
-from .globals import (
+from pygpsclient.globals import (
     CONNECTED,
     CONNECTED_FILE,
     DISCONNECTED,
-    NMEA_PROTOCOL,
-    MIXED_PROTOCOL,
-    UBX_PROTOCOL,
+    QUITONERRORDEFAULT,
 )
-from .strings import NOTCONN, SEROPENERROR, ENDOFFILE
+from pygpsclient.strings import NOTCONN, SEROPENERROR, FILEOPENERROR, ENDOFFILE
 
 
 class SerialHandler:
@@ -48,6 +47,7 @@ class SerialHandler:
         self._serial_thread = None
         self._file_thread = None
         self._connected = False
+        self._reader = None
         self._reading = False
 
     def __del__(self):
@@ -70,6 +70,7 @@ class SerialHandler:
             return
 
         try:
+
             self._serial_object = Serial(
                 serial_settings.port,
                 serial_settings.bpsrate,
@@ -81,24 +82,11 @@ class SerialHandler:
                 timeout=serial_settings.timeout,
             )
             self._serial_buffer = BufferedReader(self._serial_object)
-            self.__app.frm_banner.update_conn_status(CONNECTED)
-            self.__app.set_connection(
-                (
-                    f"{serial_settings.port}:{serial_settings.port_desc} "
-                    + f"@ {str(serial_settings.bpsrate)}"
-                ),
-                "green",
+            msg = (
+                f"{serial_settings.port}:{serial_settings.port_desc}"
+                + f"@ {str(serial_settings.bpsrate)}"
             )
-            self.__app.frm_settings.enable_controls(CONNECTED)
-            self._connected = True
-            self.start_read_thread()
-
-            if self.__app.frm_settings.datalogging:
-                self.__app.file_handler.open_logfile()
-
-            if self.__app.frm_settings.record_track:
-                self.__app.file_handler.open_trackfile()
-            self.__app.set_status("Connected", "blue")
+            self.connect_stream(self._serial_buffer, CONNECTED, msg)
 
         except (IOError, SerialException, SerialTimeoutException) as err:
             self._connected = False
@@ -123,27 +111,47 @@ class SerialHandler:
             return
 
         try:
+
             self._serial_object = open(in_filepath, "rb")
             self._serial_buffer = BufferedReader(self._serial_object)
-            self.__app.frm_banner.update_conn_status(CONNECTED_FILE)
-            self.__app.set_connection(f"{in_filepath}", "blue")
-            self.__app.frm_settings.enable_controls(CONNECTED_FILE)
-            self._connected = True
-            self.start_readfile_thread()
-
-            if self.__app.frm_settings.datalogging:
-                self.__app.file_handler.open_logfile()
-
-            if self.__app.frm_settings.record_track:
-                self.__app.file_handler.open_trackfile()
+            msg = f"{in_filepath}"
+            self.connect_stream(self._serial_buffer, CONNECTED_FILE, msg)
 
         except (IOError, SerialException, SerialTimeoutException) as err:
             self._connected = False
             self.__app.set_connection(f"{in_filepath}", "red")
-            self.__app.set_status(SEROPENERROR.format(err), "red")
+            self.__app.set_status(FILEOPENERROR.format(err), "red")
             self.__app.frm_banner.update_conn_status(DISCONNECTED)
             self.__app.frm_settings.enable_controls(DISCONNECTED)
 
+    def connect_stream(self, stream: object, status: int, msg: str):
+        """
+        Instantiate UBXReader object and start the relevant
+        serial or file reader thread.
+
+        :param object stream: serial or file stream
+        :param str status: serial or file
+        :param str msg: connection descriptor message
+        """
+
+        self._reader = UBXReader(stream, quitonerror=QUITONERRORDEFAULT)
+
+        if self.__app.frm_settings.datalogging:
+            self.__app.file_handler.open_logfile()
+
+        if self.__app.frm_settings.record_track:
+            self.__app.file_handler.open_trackfile()
+
+        self.__app.set_status("Connected", "blue")
+        self.__app.frm_banner.update_conn_status(status)
+        self.__app.frm_settings.enable_controls(status)
+        self.__app.set_connection(msg, "green")
+        self._connected = True
+        if status == CONNECTED:  # serial
+            self.start_read_thread()
+        else:  # file
+            self.start_readfile_thread()
+
     def disconnect(self):
         """
         Close serial connection.
@@ -152,6 +160,7 @@ class SerialHandler:
         if self._connected:
             try:
                 self._reading = False
+                self._reader = None
                 self._serial_object.close()
                 self.__app.frm_banner.update_conn_status(DISCONNECTED)
                 self.__app.set_connection(NOTCONN, "red")
@@ -170,41 +179,56 @@ class SerialHandler:
         self.__app.frm_settings.enable_controls(self._connected)
 
     @property
-    def port(self):
+    def port(self) -> str:
         """
         Getter for port
+
+        :return: port descriptor
+        :rtype: str
         """
 
         return self.__app.frm_settings.serial_settings().port
 
     @property
-    def connected(self):
+    def connected(self) -> int:
         """
         Getter for connection status
+
+        :return: connection status flag
+        :rtype: int
         """
 
         return self._connected
 
     @property
-    def serial(self):
+    def serial(self) -> object:
         """
         Getter for serial object
+
+        :return: serial object
+        :rtype: object
         """
 
         return self._serial_object
 
     @property
-    def buffer(self):
+    def buffer(self) -> object:
         """
         Getter for serial buffer
+
+        :return: serial buffer object
+        :rtype: object
         """
 
         return self._serial_buffer
 
     @property
-    def thread(self):
+    def thread(self) -> object:
         """
         Getter for serial thread
+
+        :return: serial reader thread
+        :rtype: object
         """
 
         return self._serial_thread
@@ -229,7 +253,7 @@ class SerialHandler:
         if self._connected:
             self._reading = True
             self.__app.frm_mapview.reset_map_refresh()
-            self._serial_thread = Thread(target=self._read_thread, daemon=True)
+            self._serial_thread = Thread(target=self._read_thread(), daemon=True)
             self._serial_thread.start()
 
     def start_readfile_thread(self):
@@ -240,7 +264,7 @@ class SerialHandler:
         if self._connected:
             self._reading = True
             self.__app.frm_mapview.reset_map_refresh()
-            self._file_thread = Thread(target=self._readfile_thread, daemon=True)
+            self._file_thread = Thread(target=self._readfile_thread(), daemon=True)
             self._file_thread.start()
 
     def stop_read_thread(self):
@@ -251,7 +275,6 @@ class SerialHandler:
         if self._serial_thread is not None:
             self._reading = False
             self._serial_thread = None
-            # self.__app.set_status(STOPDATA, "red")
 
     def stop_readfile_thread(self):
         """
@@ -261,7 +284,6 @@ class SerialHandler:
         if self._file_thread is not None:
             self._reading = False
             self._file_thread = None
-            # self.__app.set_status(STOPDATA, "red")
 
     def _read_thread(self):
         """
@@ -277,7 +299,7 @@ class SerialHandler:
         except SerialException as err:
             self.__app.set_status(f"Error in read thread {err}", "red")
         # spurious errors as thread shuts down after serial disconnection
-        except (TypeError, OSError) as err:
+        except (TypeError, OSError):
             pass
 
     def _readfile_thread(self):
@@ -299,7 +321,7 @@ class SerialHandler:
 
         if self._reading and self._serial_object is not None:
             try:
-                self._parse_data(self._serial_buffer)
+                self._parse_data()
             except SerialException as err:
                 self.__app.set_status(f"Error {err}", "red")
 
@@ -313,70 +335,46 @@ class SerialHandler:
         self.disconnect()
         self.__app.set_status(ENDOFFILE, "blue")
 
-    def _parse_data(self, ser: Serial):
+    def _parse_data(self):
         """
-        Read the binary data and direct to the appropriate
-        UBX and/or NMEA protocol handler, depending on which protocols
-        are filtered.
-
-        :param Serial ser: serial port
+        Invoke the UBXReader.read() method to read and parse the data stream
+        and direct to the appropriate UBX and/or NMEA protocol handler,
+        depending on which protocols are filtered.
         """
 
-        parsing = True
         raw_data = None
         parsed_data = None
-        byte1 = ser.read(1)  # read first byte to determine protocol
-        if len(byte1) < 1:
+        protfilter = self.__app.frm_settings.protocol
+
+        try:
+            (raw_data, parsed_data) = self._reader.read()
+            if raw_data is None:
+                raise EOFError
+        except EOFError:
             self.__master.event_generate("<<ubx_eof>>")
             return
+        except (UBXParseError, NMEAParseError) as err:
+            # log errors to console, then continue
+            self.__app.frm_console.update_console(bytes(str(err), "utf-8"), err)
+            return
 
-        while parsing:
-            filt = self.__app.frm_settings.protocol
-            byte2 = ser.read(1)
-            if len(byte2) < 1:
-                self.__master.event_generate("<<ubx_eof>>")
-                return
-            # if it's a UBX message (b'\b5\x62')
-            if byte1 == b"\xb5" and byte2 == b"\x62":
-                byten = ser.read(4)
-                if len(byten) < 4:
-                    self.__master.event_generate("<<ubx_eof>>")
-                    parsing = False
-                    break
-                clsid = byten[0:1]
-                msgid = byten[1:2]
-                lenb = byten[2:4]
-                leni = int.from_bytes(lenb, "little", signed=False)
-                byten = ser.read(leni + 2)
-                if len(byten) < leni + 2:
-                    self.__master.event_generate("<<ubx_eof>>")
-                    parsing = False
-                    break
-                plb = byten[0:leni]
-                cksum = byten[leni : leni + 2]
-                raw_data = ubt.UBX_HDR + clsid + msgid + lenb + plb + cksum
-                if filt in (UBX_PROTOCOL, MIXED_PROTOCOL):
-                    parsed_data = self.__app.ubx_handler.process_data(raw_data)
-                parsing = False
-            # if it's an NMEA message ('$G' or '$P')
-            elif byte1 == b"\x24" and byte2 in (b"\x47", b"\x50"):
-                raw_data = byte1 + byte2 + ser.readline()
-                if filt in (NMEA_PROTOCOL, MIXED_PROTOCOL):
-                    parsed_data = self.__app.nmea_handler.process_data(raw_data)
-                parsing = False
-            # else drop it like it's hot
-            else:
-                parsing = False
+        # print(f"DEBUG UBXReader._parse_data r:{raw_data} p:{parsed_data}")
+        if raw_data is None or parsed_data is None:
+            return
+        msgprot = protocol(raw_data)
+        if msgprot == ubt.UBX_PROTOCOL and msgprot & protfilter:
+            self.__app.frm_console.update_console(raw_data, parsed_data)
+            self.__app.ubx_handler.process_data(raw_data, parsed_data)
+        elif msgprot == ubt.NMEA_PROTOCOL and msgprot & protfilter:
+            self.__app.frm_console.update_console(raw_data, parsed_data)
+            self.__app.nmea_handler.process_data(raw_data, parsed_data)
+        elif msgprot == 0 and protfilter == 3:
+            # log unknown protocol headers to console, then continue
+            self.__app.frm_console.update_console(raw_data, parsed_data)
 
         # if datalogging, write to log file
         if self.__app.frm_settings.datalogging:
-            logformat = self.__app.frm_settings.logformat
-            if logformat in ("Bin", "All") and raw_data is not None:
-                self.__app.file_handler.write_logfile(raw_data)
-            if logformat in ("Hex", "Hex+Parsed", "All") and raw_data is not None:
-                self.__app.file_handler.write_logfile(raw_data.hex())
-            if logformat in ("Parsed", "Hex+Parsed", "All") and parsed_data is not None:
-                self.__app.file_handler.write_logfile(parsed_data)
+            self.__app.file_handler.write_logfile(raw_data, parsed_data)
 
     def flush(self):
         """
